#ifndef lint
static const char rcsid[] = "$Header: /net/hopkins/var/home/tristan/mpeg/dec/RCS/simple_player.c,v 1.28 1998/06/12 03:12:14 tristan Exp $";
static const char copyright[] = "Copyright (c) 1997, MpegTV LLC";
#endif


/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "mtvp/mtvp.h"

#include "forms.h"
#include "simple_player.h"
#include "simple_player_controls.h"

/*--------------------------------------------------------------------------+
| Configuration flags														|
+--------------------------------------------------------------------------*/
/*
 * Define TEST_SII only to test a SII plugin.
 * Value of TEST_SII is path of the SII dynamic library.
 */
/* #define TEST_SII	"../sii/sample_sii.so" */


/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
/* for the msg.h package */
#define MSG_PTR (&Msg)

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Globals																	|
+--------------------------------------------------------------------------*/

/* The form pointers */
FD_controls *Controls;

/*--------------------------------------------------------------------------+
| Statics																	|
+--------------------------------------------------------------------------*/

/* to communicate with player */
static PMP_STRUCT	Player;

/* for the msg.h package */
static MSG_STRUCT Msg;

/*--------------------------------------------------------------------------+
| clean_exit																|
+--------------------------------------------------------------------------*/
static void
clean_exit(int n)
{	
	MESSAGE(("(pid=%u): exiting, status = %d", (u_int)getpid(), n));

#if 0 /* Tristan Savatier - Thu Jun 11 18:13:09 PDT 1998 */
//	/* a sure way to kill the Player... */
//	
//	/* make sure we ignore SIGCHLD signals */
//	(void)signal(SIGCHLD, SIG_IGN);
//	
//	/* send SIGTERM signal to Player */
//	if (Player.pid != 0) {
//		kill(Player.pid, SIGTERM);
//	}
#endif /* 0 - Tristan Savatier - Thu Jun 11 18:13:09 PDT 1998 */
	
	/* terminate (real exit this time) */
	exit(n);
}


/*--------------------------------------------------------------------------+
| at_close_controls															|
+--------------------------------------------------------------------------*/
static int 
at_close_controls(
	FL_FORM *fl_form,
	void	*data
)
{
	/*
	 * This callback is called when the controls form is closed.
	 */
	command(NULL, COM_QUIT);
	return FL_OK;
}


/*--------------------------------------------------------------------------+
| a2g_read_command															|
+--------------------------------------------------------------------------*/
static int
a2g_read_command(int fd)
{
	PMP_HEADER	msg;		/* to hold message header from Player */ 
	PMP_DATA_A2G	data;	/* to hold message data from Player */
	
	/*
	 * This  callback is called when there is something available for
	 * reading on Player.receive_channel.msg_pipe[PMP_FD_READ]
	 * (also equal to fd in this routine).
	 *
	 * This routine should read one message from the Player
	 * and take the appropriate action.
	 *
	 * It is mostly a place holder in this example: This sample
	 * application ignores most messages.  It just checks the
	 * API version number and quits if the message A2G_COM_QUIT
	 * is received (which should not happen, by the way!).
	 */
	 
	/* read message from player */
	if (a2g_read_msg(&Player, &msg, &data) == -1) {
		FATAL_ERROR(("a2g_read_msg: error reading msg"));
	}

	/* trace message if (Msg_select & MSG_INFO) */
	{
		char buf[PMP_PRINT_MSG_BUF_SIZE];
		MESSAGE(("a2g_read_command: received msg: %s", pmp_print_msg(&msg,  (const char *)&data, buf)));
	}
	
	switch ((A2G_COM)msg.type) {
		case A2G_COM_STATE:
			/* show Player's state in the form title */
			switch (data.data_generic.data_int) {
			case STATE_STOPPED:
				fl_set_form_title(Controls->controls, "Simple Player (Stopped)");
				break;
			case STATE_PAUSED:
				fl_set_form_title(Controls->controls, "Simple Player (Paused)");
				break;
			case STATE_PLAYING:
				fl_set_form_title(Controls->controls, "Simple Player (Playing)");
				break;
			default:
				FATAL(("unknow State (should not happen!)"));
			}
			break;
#ifdef TEST_SII
		case A2G_COM_SII_FEEDBACK:
			printf("received A2G_COM_SII_FEEDBACK, size = %u, data = \"%s\"\n", msg.size, (const char *)&data);
			break;
#endif /* TEST_SII */
		case A2G_COM_QUIT:
			/*
			 * Player sends A2G_COM_QUIT after receiving COM_QUIT and before terminating.
			 */
			/* remove this callback (g2a_cleanup will close this pipe) */
			fl_remove_io_callback(Player.receive_channel.msg_pipe[PMP_FD_READ], FL_READ, (void (*)(int, void *))a2g_read_command);

			/* g2a_cleanup() should be called to close pipes to the (now defunct) player */
			g2a_cleanup(&Player);
			
			clean_exit(EXIT_SUCCESS);
			
			break;
		default:
			/* msgs not understood should be ignored */
			break;
	}

	return 0;
}



/*--------------------------------------------------------------------------+
| command																	|
+--------------------------------------------------------------------------*/
void
command(
	FL_OBJECT *fl_object,
	long param
)
{
	COMMAND	com = param;
	
	/*
	 * This callback is called when clicking on any of the Xforms graphic controls
	 * of the control panel.  The param indicates which control was used.
	 */
	
	MESSAGE(("executing command call-back"));
	
	switch (com) {
		case ACTION_QUIT:

			/*
			 * Tell Player to terminate by sending the COM_QUIT command. You could also
			 * send the SIGTERM signal to the process (pid = Player.pid), but
			 * we recommand to use COM_QUIT. Using a signal may cause problems
			 * when using a multithreaded version of mtvp.
			 */
			g2a_send_void(&Player, COM_QUIT, 0);
			
			break;
		case ACTION_PLAY:
		
			/* tell Player to start playing */
			g2a_send_int(&Player, COM_STATE, 0, STATE_PLAYING);
			
			break;
		case ACTION_STEP:
		
			/* tell Player to Step one frame */
			/* Note: if player was playing, state changes automatically to STATE_PAUSED */
			g2a_send_void(&Player, COM_STEP, 0);
			
			break;
		case ACTION_REWIND:
		
			/* tell Player to Rewind */
			g2a_send_int(&Player, COM_SEEK, 0, 0);
			
			break;
		case ACTION_OPEN:
			{
				/* get file name with standard Xforms file selector */
				const char *filename = fl_show_fselector ("Open",
										NULL,
										"*.mpg",
										NULL);
				if (filename != NULL) {
			
					/* tell Player to open a new file */
#ifdef TEST_SII
					g2a_send_custom(&Player, COM_SII_OPEN, 0, filename, strlen(filename) + 1);
#else /* not TEST_SII */
					g2a_send_string(&Player, COM_OPEN_FILE, 0, filename);
#endif /* not TEST_SII */
				}
			}
			break;
		default:
			/* this hould not happen! */
			FATAL_ERROR(("command callback does not expect command %d", com));
			break;
	}
}

/*--------------------------------------------------------------------------+
| init_form																	|
+--------------------------------------------------------------------------*/
static void
init_form(
	int		argc,
	const char **argv
)
{	
	if (fl_initialize (&argc, (char **)argv, "Simple Player (Stopped)", 0, 0) == NULL) {
		FATAL_ERROR(("cannot initialize xform"));
	}

	if ((Controls = create_form_controls()) == NULL) {
		FATAL_ERROR(("cannot create Controls form"));
	}
	
	fl_set_form_atclose(Controls->controls, at_close_controls, NULL);

	fl_show_form(Controls->controls, FL_PLACE_GEOMETRY, FL_FULLBORDER, "Simple Player");

	MESSAGE(("adding io call-back"));

	fl_add_io_callback(Player.receive_channel.msg_pipe[PMP_FD_READ], FL_READ, (void (*)(int, void *))a2g_read_command, NULL);

}



#if 0 /* Tristan Savatier - Thu Jun 11 18:13:21 PDT 1998 */
///*--------------------------------------------------------------------------+
//| handler_sigchld															|
//+--------------------------------------------------------------------------*/
///*ARGSUSED*/
//static void
//handler_sigchld(sig)
//	int     sig;
//{
//	/*
//	 * Note: If you want to call any Xlib or Xforms routine from
//	 * a signal handler, you must register it using
//	 * fl_add_signal_callback() so that
//	 * the signal handler will be executed in the proper context
//	 * in the Xforms event loop.  Please refer to the Xforms
//	 * manual, Signals section.
//	 *
//	 * This particular signal handler does not
//	 * call any Xlib or Xforms routine, so it is safe
//	 * to install it with signal() or sigaction().
//	 */
//
//	MESSAGE(("(pid=%u): received signal %d, child process %u died, exiting...", (u_int)getpid(), sig, (u_int)Player.pid));
//	clean_exit(EXIT_FAILURE);
//}
#endif /* 0 - Tristan Savatier - Thu Jun 11 18:13:21 PDT 1998 */


/*--------------------------------------------------------------------------+
| main																		|
+--------------------------------------------------------------------------*/
/*ARGSUSED*/
int
main(
	int argc,
	const char **argv,
	const char **envp
)
{
	const char	**p, **q;
	int		n;

	/*
	 * Set Msg.select == -1 to print all the warnings and messages.
	 * See msg.h for the MESSAGE() and WARNING() macros definitions.
	 */
	 
	/* Msg.select = -1; */
	
#if 0 /* Tristan Savatier - Thu Jun 11 18:13:33 PDT 1998 */
//	{
//		struct sigaction sigact;
//		
//		sigact.sa_handler = handler_sigchld;
//		sigemptyset(&sigact.sa_mask);
//		sigact.sa_flags = SA_NOCLDSTOP;
//		if (sigaction(SIGCHLD, &sigact, NULL) != 0) {
//			FATAL_ERROR(("can't set handler for SIGCHLD"));
//		}
//	}
#endif /* 0 - Tristan Savatier - Thu Jun 11 18:13:33 PDT 1998 */

	MESSAGE(("started, pid = %u", (u_int)getpid()));
	
	/*
	 * Start the player
	 */
	
	{
		static const char *mtvp_argv[] = {
			"../../bin/mtvp",	/* name of the player (if no / in pathname, will search in PATH) */
			/* "-w", */	/* possible options */
			NULL,	/* must be NULL-terminated */
		};

		/*
		 * First parameter of g2a_init() is a ptr to a NULL-terminated array
		 * of strings.
		 * The first string is the name of the Player ("mtvp").
		 * The following strings are possible parameters.
		 * If the first string is a full pathname (starting with "/")
		 * g2a_init() will run this program as "the Player".
		 *
		 * If the first string is not a full pathname (starting with "/")
		 * g2a_init() will look for the Player in the following places:
		 * - in the directory where argv[0] is located (if argv[0] contains a '/')
		 * - in the current directory (".")
		 * - in the PATH
		 *
		 * Typically first string is "mtvp", if the player was installed
		 * under this name.
		 *
		 * Second parameter of g2a_init() is a string that should be
		 * the argv[0] from the calling program.
		 */
	
		if (g2a_init(&Player, mtvp_argv, argv[0]) == -1) {
			if (errno == 0) {
				FATAL_ERROR(("cannot start player (%s) - probably mtvp API version mismatch", mtvp_argv[0]));
			} else if (errno == EPIPE) {
				FATAL_ERROR(("cannot start player (%s) - check that player is in your PATH!", mtvp_argv[0]));
			} else {
				FATAL_ERROR(("cannot start player (%s) - could not fork", mtvp_argv[0]));
			}
		}
	}
	
	/* initialize the form */
	init_form(argc, argv);
	
#ifdef TEST_SII
	/* link custom stream interface */
	g2a_send_string(&Player, COM_SII_LINK, 0, TEST_SII);
#endif
	
	/* if one argument was passed, assume that it is a file to open */
	if (argc == 2) {
#ifdef TEST_SII
		/* open stream with custom stream interface */
		g2a_send_custom(&Player, COM_SII_OPEN, 0, argv[1], strlen(argv[1]) + 1);
		
#else /* not TEST_SII */

		/* open stream with standard file interface */
		g2a_send_string(&Player, COM_OPEN_FILE, 0, argv[1]);
		
#endif /* not TEST_SII */
	} else if (argc > 2) {
		fprintf(stderr, "usage: simple_player [file.mpg]\n");
		clean_exit(EXIT_FAILURE);		
	}

	MESSAGE(("entering event loop"));

	/* enter event loop */
	fl_do_forms();
	
	MESSAGE(("What?!? fl_do_forms() returned! (should never happen)"));

	/* terminated successfully */
	clean_exit(EXIT_SUCCESS);

	/* so that lint does not complain! */
	return 0;
}


/* eof */
