#ifndef lint
static char rcsid[] = "$Header: /net/hopkins/var/home/tristan/mpeg/dec/RCS/store.c,v 1.6 1997/10/21 10:29:46 tristan Exp $";
static char copyright[] = "Copyright (c) 1998, MpegTV LLC";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "mtvp/misc.h"
#include "mtvp/msg.h"

#include "yuvtorgb.h"

/* for the msg.h package */
extern MSG_STRUCT Msg;

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
#define STORE_BUFFER_SIZE 4096

/* for the msg.h package */
#define MSG_PTR (&Msg)

/*--------------------------------------------------------------------------+
| Statics																	|
+--------------------------------------------------------------------------*/

static u_char Store_buffer[STORE_BUFFER_SIZE];
static u_char *Store_ptr;
static int Store_fd;
static Bool Store_error;
static int Store_errno;


/*--------------------------------------------------------------------------+
| store_flush_buffer														|
+--------------------------------------------------------------------------*/
static void
store_flush_buffer()
{
	if (write (Store_fd, Store_buffer, (size_t)(Store_ptr - Store_buffer)) != Store_ptr - Store_buffer) {
		Store_error = True;
		Store_errno = errno;
	}
	Store_ptr = Store_buffer;
}

/*--------------------------------------------------------------------------+
| store_putbyte																|
+--------------------------------------------------------------------------*/
static INLINE void
store_putbyte (c)
	 int     c;
{
	*Store_ptr++ = c;

	if (Store_ptr == Store_buffer + STORE_BUFFER_SIZE) {
		store_flush_buffer();
	}
}

/*--------------------------------------------------------------------------+
| store_putword																|
+--------------------------------------------------------------------------*/
static INLINE void
store_putword (w)
	 int     w;
{
	store_putbyte (w);
	store_putbyte (w >> 8);
}



/*
 * store as PPM (PBMPLUS) or uncompressed Truevision TGA ('Targa') file
 */
int 
store_ppm_tga (lum, ucr, ucb, filename, rows, cols, tgaflag)
	 const u_char *lum;
	 const u_char *ucr;
	 const u_char *ucb;
	 const char *filename;
	 int     rows, cols;
	 Bool     tgaflag;
{
	int		i;
	int     x,
	        y;
	static unsigned char tga24[14] =
	{0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 24, 32};
	char    header[256];
	
#if 0 /* Tristan Savatier - Wed Feb 19 02:13:50 PST 1997 */
//	if (!Ready) {
//		errno = 0;
//		return -1;
//	}
#endif /* 0 - Tristan Savatier - Wed Feb 19 02:13:50 PST 1997 */

	MESSAGE(("Saving frame in PPM file %s\n", filename));

	if ((Store_fd = open (filename, O_CREAT | O_EXCL | O_WRONLY | O_BINARY, 0666)) == -1) {
		MESSAGE_ERROR(("Couldn't create %s\n", filename));
		return -1;
	}
	Store_ptr = Store_buffer;
	Store_error = False;

	if (tgaflag) {
		/* TGA header */
		for (i = 0; i < 12; i++) {
			store_putbyte (tga24[i]);
		}

		store_putword (cols);
		store_putword (rows);
		store_putbyte (tga24[12]);
		store_putbyte (tga24[13]);
	} else {
		/* PPM header */
		sprintf (header, "P6\n%d %d\n255\n", cols, rows);
		assert(strlen(header) < sizeof header);

		for (i = 0; header[i] != 0; i++) {
			store_putbyte (header[i]);
		}
	}

	for (y = 0; y < rows; y++) {
		x = cols / 2;

		do {
    		u_char r, g, b;

			convert_color(lum[0], ucr[0], ucb[0], &r, &g, &b);

			if (tgaflag) {
				store_putbyte (b);
				store_putbyte (g);
				store_putbyte (r);
			} else {
				store_putbyte (r);
				store_putbyte (g);
				store_putbyte (b);
			}

			convert_color(lum[1], ucr[0], ucb[0], &r, &g, &b);

			if (tgaflag) {
				store_putbyte (b);
				store_putbyte (g);
				store_putbyte (r);
			} else {
				store_putbyte (r);
				store_putbyte (g);
				store_putbyte (b);
			}
			lum += 2; ucr++; ucb++;

		} while (--x > 0);

		if (is_even(y)) {
			ucr -= cols / 2;
			ucb -= cols / 2;
		}
	}

	if (Store_ptr != Store_buffer) {
		store_flush_buffer();
	}

	close (Store_fd);
	
	if (Store_error) {
		errno = Store_errno;
		return -1;
	} else {
		return 0;
	}
}
